<?php

namespace EventPortal\Domain\Agenda\Actions;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Agenda\SessionInput;
use EventPortal\Domain\Agenda\SessionOverlap;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use Illuminate\Support\Arr;

class CreateSession
{
    public function execute(Pathway $pathway, Event $event, array $input = []): Session
    {
        $input = SessionInput::from($input)
            ->forceDate($pathway->agenda->date)
            ->formatForDatabase();

        $form = Form::make('session.create')
            ->model(new Session)
            ->useAlternativeNames()
            ->context($pathway, $event)
            ->validate($input);

        $overlaps = Session::query()
            ->forPathway($pathway)
            ->completelyOverlaps($input['starting_at'], $input['ending_at'])
            ->exists();

        SessionOverlap::throwValidationExceptionIfOverlapping($form->getValidator(), $overlaps);

        $session = $form->handle($input)->getModel();

        if (Arr::has($input, 'event_speaker_id')) {
            $eventSpeakerIds = collect($input['event_speaker_id'])->pluck('value')->filter();

            $session->eventSpeakers()->attach($eventSpeakerIds);
        }

        return $session;
    }
}
